@@ -21,15 +21,11 @@ public:
 
   libremidi::API get_current_api() const noexcept override;
 
-  bool open_port(unsigned int portNumber, std::string_view);
-  bool open_port(const output_port& p, std::string_view) override;
-  bool open_virtual_port(std::string_view) override;
-  void close_port() override;
+  stdx::error open_port(int portNumber, std::string_view);
+  stdx::error open_port(const output_port& p, std::string_view) override;
+  stdx::error close_port() override;
 
-  void set_client_name(std::string_view clientName) override;
-  void set_port_name(std::string_view portName) override;
-
-  void send_message(const unsigned char* message, size_t size) override;
+  stdx::error send_message(const unsigned char* message, size_t size) override;
 
 private:
   int portNumber_{-1};
