@@ -1,11 +1,15 @@
 #pragma once
+#include <ossia/detail/config.hpp>
 
-#if __has_include(<dlfcn.h>)
-#include <ossia/detail/fmt.hpp>
-
+#ifdef _WIN32
+#include <windows.h>
+#else
 #include <dlfcn.h>
-
+#endif
 #include <stdexcept>
+#include <string>
+#include <string_view>
+#include <vector>
 
 namespace ossia
 {
@@ -14,10 +18,14 @@ class dylib_loader
 public:
   explicit dylib_loader(const char* const so)
   {
+#ifdef _WIN32
+    impl = (void*)LoadLibraryA(so);
+#else
     impl = dlopen(so, RTLD_LAZY | RTLD_LOCAL | RTLD_NODELETE);
+#endif
     if(!impl)
     {
-      throw std::runtime_error(fmt::format("{}: not found. ", so));
+      throw std::runtime_error(std::string(so) + ": not found. ");
     }
   }
 
@@ -28,12 +36,16 @@ public:
 
     for(const auto so : sos)
     {
+#ifdef _WIN32
+      impl = (void*)LoadLibraryA(so.data());
+#else
       impl = dlopen(so.data(), RTLD_LAZY | RTLD_LOCAL | RTLD_NODELETE);
+#endif
       if(impl)
         return;
     }
 
-    throw std::runtime_error(fmt::format("{}: not found. ", sos[0]));
+    throw std::runtime_error(std::string(sos[0]) + ": not found. ");
   }
 
   dylib_loader(const dylib_loader&) noexcept = delete;
@@ -55,14 +67,22 @@ public:
   {
     if(impl)
     {
+#ifdef _WIN32
+      FreeLibrary((HMODULE)impl);
+#else
       dlclose(impl);
+#endif
     }
   }
 
   template <typename T>
   T symbol(const char* const sym) const noexcept
   {
+#ifdef _WIN32
+    return (T)GetProcAddress((HMODULE)impl, sym);
+#else
     return (T)dlsym(impl, sym);
+#endif
   }
 
   operator bool() const { return bool(impl); }
@@ -71,4 +91,3 @@ private:
   void* impl{};
 };
 }
-#endif
