@@ -6,6 +6,7 @@
 #include <boost/asio/ip/udp.hpp>
 #include <boost/asio/local/datagram_protocol.hpp>
 #include <boost/asio/placeholders.hpp>
+#include <boost/asio/strand.hpp>
 #include <boost/asio/write.hpp>
 
 #include <nano_signal_slot.hpp>
@@ -18,15 +19,22 @@ class udp_receive_socket
   using proto = boost::asio::ip::udp;
 
 public:
+  udp_receive_socket(boost::asio::io_context& ctx)
+      : m_context{ctx}
+      , m_socket{boost::asio::make_strand(ctx)}
+  {
+  }
+
   udp_receive_socket(const socket_configuration& conf, boost::asio::io_context& ctx)
       : m_context{ctx}
       , m_endpoint{boost::asio::ip::make_address(conf.host), conf.port}
-      , m_socket{ctx}
+      , m_socket{boost::asio::make_strand(ctx)}
   {
   }
 
   ~udp_receive_socket() = default;
 
+  void assign(int sock) { m_socket.assign(boost::asio::ip::udp::v4(), sock); }
   void open()
   {
     m_socket.open(boost::asio::ip::udp::v4());
@@ -89,7 +97,7 @@ public:
   udp_send_socket(const socket_configuration& conf, boost::asio::io_context& ctx)
       : m_context{ctx}
       , m_endpoint{conf.broadcast ? boost::asio::ip::address_v4::broadcast() : boost::asio::ip::make_address(conf.host), conf.port}
-      , m_socket{ctx}
+      , m_socket{boost::asio::make_strand(ctx)}
   {
   }
 
@@ -98,7 +106,7 @@ public:
       boost::asio::io_context& ctx)
       : m_context{ctx}
       , m_endpoint{host, port}
-      , m_socket{ctx}
+      , m_socket{boost::asio::make_strand(ctx)}
   {
   }
 
