@@ -25,6 +25,7 @@
 #include <QHeaderView>
 #include <QLabel>
 #include <QPushButton>
+#include <QRadioButton>
 #include <QSerialPortInfo>
 #include <QTableWidget>
 #include <QTimer>
@@ -1223,38 +1224,31 @@ ArtnetProtocolSettingsWidget::ArtnetProtocolSettingsWidget(QWidget* parent)
 {
   m_deviceNameEdit = new State::AddressFragmentLineEdit{this};
   m_deviceNameEdit->setText("Artnet");
+  checkForChanges(m_deviceNameEdit);
 
   m_host = new QComboBox{this};
   m_host->setEditable(true);
-  m_host->addItems(score::list_ipv4());
+  checkForChanges(m_host);
 
   m_rate = new QSpinBox{this};
   m_rate->setRange(0, 44);
   m_rate->setValue(20);
 
   m_universe = new QSpinBox{this};
-  m_universe->setRange(1, 65539);
+  m_universe->setRange(0, 65539);
 
   m_transport = new QComboBox{this};
-  m_transport->addItems({"ArtNet", "E1.31 (sACN)", "DMX USB PRO"});
+  m_transport->addItems({"ArtNet", "E1.31 (sACN)", "DMX USB PRO", "DMX USB PRO Mk2"});
+  checkForChanges(m_transport);
+
+  m_source = new QRadioButton{tr("Send DMX"), this};
+  m_sink = new QRadioButton{tr("Receive DMX"), this};
+  m_source->setChecked(true);
 
   connect(
       m_transport, qOverload<int>(&QComboBox::currentIndexChanged), this,
-      [this](int idx) {
-    m_host->clear();
-    switch(idx)
-    {
-      case 0:
-      case 1:
-        m_host->addItems(score::list_ipv4());
-        break;
-      case 2: {
-        for(auto port : QSerialPortInfo::availablePorts())
-          m_host->addItem(port.portName());
-        break;
-      }
-    }
-      });
+      &ArtnetProtocolSettingsWidget::updateHosts);
+  updateHosts(m_transport->currentIndex());
 
   auto layout = new QFormLayout;
   layout->addRow(tr("Name"), m_deviceNameEdit);
@@ -1263,6 +1257,11 @@ ArtnetProtocolSettingsWidget::ArtnetProtocolSettingsWidget(QWidget* parent)
   layout->addRow(tr("Transport"), m_transport);
   layout->addRow(tr("Interface"), m_host);
 
+  auto radiolay = new QHBoxLayout{};
+  radiolay->addWidget(m_source);
+  radiolay->addWidget(m_sink);
+  layout->addRow(tr("Mode"), radiolay);
+
   m_fixturesWidget = new QTableWidget;
   layout->addRow(m_fixturesWidget);
 
@@ -1312,6 +1311,43 @@ ArtnetProtocolSettingsWidget::ArtnetProtocolSettingsWidget(QWidget* parent)
   setLayout(layout);
 }
 
+void ArtnetProtocolSettingsWidget::updateHosts(int idx)
+{
+
+  m_host->clear();
+  switch(idx)
+  {
+    case 0: {
+      auto ips = score::list_ipv4();
+      ips.removeAll("0.0.0.0");
+      m_host->addItems(ips);
+      m_host->setCurrentIndex(0);
+      m_universe->setRange(0, 16);
+      break;
+    }
+    case 1:
+      m_host->addItems(score::list_ipv4());
+      m_host->setCurrentIndex(0);
+      m_universe->setRange(1, 65539);
+      break;
+    case 2: {
+      m_universe->setRange(0, 0);
+      for(const auto& port : QSerialPortInfo::availablePorts())
+        m_host->addItem(port.portName());
+      break;
+    }
+    case 3: {
+      m_universe->setRange(0, 1);
+      for(const auto& port : QSerialPortInfo::availablePorts())
+        m_host->addItem(port.portName());
+      break;
+    }
+  }
+
+  if(m_host->currentText().isEmpty())
+    m_host->setCurrentIndex(0);
+}
+
 void ArtnetProtocolSettingsWidget::updateTable()
 {
   while(m_fixturesWidget->rowCount() > 0)
@@ -1337,6 +1373,7 @@ void ArtnetProtocolSettingsWidget::updateTable()
     row++;
   }
 }
+
 ArtnetProtocolSettingsWidget::~ArtnetProtocolSettingsWidget() { }
 
 Device::DeviceSettings ArtnetProtocolSettingsWidget::getSettings() const
@@ -1360,10 +1397,15 @@ Device::DeviceSettings ArtnetProtocolSettingsWidget::getSettings() const
     case 2:
       settings.transport = ArtnetSpecificSettings::DMXUSBPRO;
       break;
+    case 3:
+      settings.transport = ArtnetSpecificSettings::DMXUSBPRO_Mk2;
+      break;
   }
 
   settings.rate = this->m_rate->value();
   settings.universe = this->m_universe->value();
+  settings.mode = this->m_source->isChecked() ? ArtnetSpecificSettings::Source
+                                              : ArtnetSpecificSettings::Sink;
   s.deviceSpecificSettings = QVariant::fromValue(settings);
 
   return s;
@@ -1374,8 +1416,34 @@ void ArtnetProtocolSettingsWidget::setSettings(const Device::DeviceSettings& set
   m_deviceNameEdit->setText(settings.name);
   const auto& specif = settings.deviceSpecificSettings.value<ArtnetSpecificSettings>();
   m_fixtures = specif.fixtures;
+
+  switch(specif.transport)
+  {
+    case ArtnetSpecificSettings::ArtNet:
+    case ArtnetSpecificSettings::ArtNetV2:
+      m_transport->setCurrentIndex(0);
+      break;
+    case ArtnetSpecificSettings::E131:
+      m_transport->setCurrentIndex(1);
+      break;
+    case ArtnetSpecificSettings::DMXUSBPRO:
+      m_transport->setCurrentIndex(2);
+      break;
+    case ArtnetSpecificSettings::DMXUSBPRO_Mk2:
+      m_transport->setCurrentIndex(3);
+      break;
+  }
+
   m_rate->setValue(specif.rate);
   m_universe->setValue(specif.universe);
+  m_host->setCurrentText(specif.host);
+  if(m_host->currentText().isEmpty())
+    updateHosts(m_transport->currentIndex());
+
+  if(specif.mode == ArtnetSpecificSettings::Source)
+    m_source->setChecked(true);
+  else
+    m_sink->setChecked(true);
   updateTable();
 }
 }
