@@ -69,7 +69,13 @@ struct ArtnetSpecificSettings
     E131,
     DMXUSBPRO,
     ArtNetV2, // Artnet:/{}
+    DMXUSBPRO_Mk2,
   } transport{ArtNetV2};
+  enum
+  {
+    Source, // score sends DMX
+    Sink    // score receives DMX
+  } mode;
 };
 }
 
